import logging

import asyncio

from ace_backend.sync import message
from ace_backend.lib import pg


logging.basicConfig(level=logging.DEBUG)


async def main_impl() -> None:
    pool = await pg.get_connection_pool()

    tasks = [message.sync_messages(pool)]

    await asyncio.gather(*tasks)


def main():
    asyncio.run(main_impl())


if __name__ == '__main__':
    main()



import dataclasses
import datetime as dt
from typing import Optional
import uuid

from aiohttp import web
import aiohttp_session

from ace_backend.sync import message as message_module


SYNC_TTL_S = 600


@dataclasses.dataclass(frozen=True)
class Response:
    user_id: Optional[str]
    sync_message: Optional[str]
    avatar: Optional[str]


def _set_sync_message(session: aiohttp_session.Session) -> None:
    session['user_id_sync'][
        'sync_message'
    ] = f'%%technical message for tuzach in/net user_id sync: {uuid.uuid4().hex}%%'
    session.changed()


def _set_user_id(session: aiohttp_session.Session, user_id: str) -> None:
    session['user_id'] = user_id
    session['user_id_sync']['last_sync_timestamp'] = dt.datetime.now().timestamp()
    del session['user_id_sync']['sync_message']


async def _is_sync_needed(
    request: web.Request, session: aiohttp_session.Session
) -> bool:
    if 'user_id' not in session:
        return True

    now = dt.datetime.now()
    last_sync_timestamp = dt.datetime.fromtimestamp(
        session['user_id_sync']['last_sync_timestamp']
    )
    if (now - last_sync_timestamp).total_seconds() < SYNC_TTL_S:
        return False

    rows = await request.app['pool'].fetch(
        '''
        SELECT sm.user_id AS user_id
        FROM sync.message AS sm
        INNER JOIN ace.message AS am
            ON sm.text = am.text AND sm.time > am.created_at - INTERVAL '5 MINUTES'
        WHERE am.id = (
            SELECT id
            FROM ace.message
            WHERE ace_session_id = $1 AND created_at < NOW() - INTERVAL '1 MINUTE'
            ORDER BY created_at DESC
            LIMIT 1
        )
        ''',
        session.identity,
    )
    user_ids = [row['user_id'] for row in rows]

    return bool(user_ids) and session['user_id'] not in user_ids


async def _do_sync(request: web.Request, session: aiohttp_session.Session) -> None:
    pg_pool = request.app['pool']
    await message_module.do_sync(pg_pool, request.app['client_session'])

    user_id = await pg_pool.fetchval(
        '''
        SELECT user_id
        FROM sync.message
        WHERE text = $1
        ORDER BY external_id
        LIMIT 1
        ''',
        session['user_id_sync']['sync_message'],
    )

    if user_id is not None:
        _set_user_id(session, user_id)

        return

    _set_sync_message(session)


async def handle(request: web.Request) -> web.Response:
    session = await aiohttp_session.get_session(request)

    if 'user_id_sync' not in session:
        session['user_id_sync'] = {}

    if 'sync_message' in session['user_id_sync']:
        await _do_sync(request, session)
    elif await _is_sync_needed(request, session):
        _set_sync_message(session)

    return web.json_response(
        dataclasses.asdict(
            Response(
                user_id=session.get('user_id'),
                sync_message=session['user_id_sync'].get('sync_message'),
                avatar=None,
            )
        )
    )



import os

from redis import asyncio as aioredis


async def get_connection() -> aioredis.Redis:
    redis_host = os.environ.get('ACE_REDIS_HOST') or 'redis'

    return await aioredis.from_url(f'redis://{redis_host}:6379')



import dataclasses
from typing import List
from typing import Optional

from aiohttp import web
import aiohttp_session

from ace_backend.sync import message as message_module


@dataclasses.dataclass(frozen=True)
class Picture:
    name: str
    imgurl: str
    thumburl: str
    filedata: str
    imgw: str
    imgh: str
    thumbw: str
    thumbh: str


@dataclasses.dataclass(frozen=True)
class Message:
    id: str
    type: str
    time: str
    text: str
    user_id: str
    avatar: Optional[str]
    picture: Optional[Picture]


@dataclasses.dataclass(frozen=True)
class Response:
    data: List[Message]
    user_count: int


async def _get_messages(request: web.Request, last: int) -> List[Message]:
    rows = await request.app['pool'].fetch(
        '''
        SELECT 
            m.external_id AS m_external_id,
            m.type AS m_type,
            m.time AS m_time,
            m.text AS m_text,
            m.user_id AS m_user_id,
            m.avatar AS m_avatar,
            p.name AS p_name,
            p.imgurl AS p_imgurl,
            p.thumburl AS p_thumburl,
            p.filedata AS p_filedata,
            p.imgw AS p_imgw,
            p.imgh AS p_imgh,
            p.thumbw AS p_thumbw,
            p.thumbh AS p_thumbh
        FROM sync.message AS m
        LEFT JOIN sync.picture AS p
            ON m.picture_id = p.id
        WHERE m.external_id > $1
        ORDER BY m.external_id DESC
        LIMIT 50
    ''',
        last,
    )

    data = []
    for row in reversed(rows):
        picture = None
        if row['p_name']:
            picture = Picture(
                name=row['p_name'],
                imgurl=row['p_imgurl'],
                thumburl=row['p_thumburl'],
                filedata=row['p_filedata'],
                imgw=row['p_imgw'],
                imgh=row['p_imgh'],
                thumbw=row['p_thumbw'],
                thumbh=row['p_thumbh'],
            )

        message = Message(
            id=str(row['m_external_id']),
            type=row['m_type'],
            time=row['m_time'].isoformat(),
            text=row['m_text'],
            user_id=row['m_user_id'],
            avatar=row['m_avatar'],
            picture=picture,
        )

        data.append(message)

    return data


async def _get_user_count(
    request: web.Request, session: aiohttp_session.Session
) -> int:
    if session.identity:
        await request.app['pool'].execute(
            '''
            INSERT INTO ace.touch
                (
                    ace_session_id
                )
            VALUES
                (
                    $1
                )
            ON CONFLICT (ace_session_id) DO UPDATE
            SET last_seen_at = NOW()
        ''',
            session.identity,
        )

    return await request.app['pool'].fetchval(
        '''
        SELECT COALESCE(COUNT(DISTINCT ace_session_id), 0) AS user_count
        FROM ace.touch
        WHERE last_seen_at > NOW() - INTERVAL '1 MINUTE'
    '''
    )


async def handle(request: web.Request) -> web.Response:
    last = int(request.query['last'])
    force_sync = bool(request.query.get('force_sync'))
    session = await aiohttp_session.get_session(request)

    if force_sync:
        await message_module.do_sync(request.app['pool'], request.app['client_session'])

    data = await _get_messages(request, last)
    user_count = await _get_user_count(request, session)

    return web.json_response(
        dataclasses.asdict(Response(data=data, user_count=user_count))
    )



import logging

import aiohttp
from aiohttp import web
import aiohttp_cors
import aiohttp_session
import aiohttp_session.redis_storage

from ace_backend.lib import pg
from ace_backend.lib import redis
from ace_backend.server.handlers import v1_message_list_get
from ace_backend.server.handlers import v1_message_post
from ace_backend.server.handlers import v1_topic_get
from ace_backend.server.handlers import v1_whoami_get


logging.basicConfig(level=logging.DEBUG)


async def init_app():
    app = web.Application()

    app['pool'] = await pg.get_connection_pool()
    app['client_session'] = aiohttp.ClientSession(raise_for_status=True)

    app.add_routes(
        [
            web.get('/v1/message/list', v1_message_list_get.handle),
            web.post('/v1/message', v1_message_post.handle),
            web.get('/v1/whoami', v1_whoami_get.handle),
            web.get('/v1/topic', v1_topic_get.handle),
        ]
    )

    cors = aiohttp_cors.setup(
        app,
        defaults={
            'http://localhost:3000': aiohttp_cors.ResourceOptions(
                allow_credentials=True, expose_headers="*", allow_headers="*"
            )
        },
    )

    for route in list(app.router.routes()):
        cors.add(route)

    storage = aiohttp_session.redis_storage.RedisStorage(
        await redis.get_connection(),
        cookie_name='ace_session',
        max_age=86400 * 30,
        secure=True,
    )
    aiohttp_session.setup(app, storage)

    return app


def main():
    web.run_app(init_app())


if __name__ == '__main__':
    main()



import dataclasses

from aiohttp import web


@dataclasses.dataclass(frozen=True)
class Response:
    topic: str


async def handle(request: web.Request) -> web.Response:
    topic = await request.app['pool'].fetchval(
        '''
        SELECT text
        FROM ace.topic
        WHERE created_at < NOW()
        ORDER BY created_at DESC
        LIMIT 1
    '''
    )

    if topic is None:
        topic = 'tuzach.in mirror'

    return web.json_response(dataclasses.asdict(Response(topic=topic)))



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from aiohttp import web
import marshmallow


T = TypeVar('T', bound=Any)


async def deser_request_body(request: web.Request, klass: Type[T]) -> T:
    request_json = await request.json()

    try:
        request_body = klass.Schema().load(request_json)
    except marshmallow.exceptions.ValidationError as exc:
        raise web.HTTPBadRequest(reason=f'Request schema validation failed: {exc}')

    return request_body


def ser_response_body(response: T) -> Dict:
    return response.Schema().dump(response)



import os

import asyncpg


async def get_connection_pool() -> asyncpg.Pool:
    return await asyncpg.create_pool(
        user='ace',
        password='ace',
        database='ace',
        host=os.environ.get('ACE_PG_HOST') or 'pg',
    )



import dataclasses
import logging

from aiohttp import web
import aiohttp_session
import marshmallow_dataclass as mdataclasses

from ace_backend.lib import server

logger = logging.getLogger(__name__)


@mdataclasses.add_schema
@dataclasses.dataclass
class Request:
    text: str


async def handle(request: web.Request) -> web.Response:
    request_body = await server.deser_request_body(request, Request)
    session = await aiohttp_session.get_session(request)

    await request.app['pool'].execute(
        '''
        INSERT INTO ace.message
            (
                ace_session_id,
                text
            )
        VALUES
            (
                $1,
                $2
            )
    ''',
        session.identity,
        request_body.text,
    )

    return web.Response()
